var path = require('path');
var Promise = require('bluebird');
var Request = require(path.join(__dirname,'Request'));
var misc = require(path.join(__dirname,'misc'));
var Teams = require(path.join(__dirname,'Teams'));
var getDB = require(path.join(__dirname,'getDBConnection'));

var oldSeasonScheduleArrayBuilder = function(season) {
  gamesCount = 1;
  /* Declaring a new Array(N) makes an object with length N but doesn't actually
    declare the key for the values (where the key in the expression array[0] is 0), 
    so .map doesn't even see these keys, so even tho it's a length N array it has 0 keys.
    Array.apply is a cool trick to mitigate this problem, see: 
    //http://stackoverflow.com/questions/18947892/creating-range-in-javascript-strange-syntax/18949651#18949651*/
  retArray = Array.apply(null, {length: 1230})
    .map(function() {
      return {id: misc.pad(gamesCount++, 4), date: undefined};
    });
  return retArray;
}

var seasonScheduleArrayBuilder = function(season) {
  teams = new Teams(season);
  var scheduleURL = 'http://live.nhl.com/GameData/SeasonSchedule-'+season+'.json';

  return Request({
    //make a get request to nhl.com, to get the specified file
    method: 'GET',
    host: 'live.nhl.com',
    port: 80,
    path: scheduleURL
  }).then(function(data) {
    var gamesList = JSON.parse(data.body), 
      today = misc.getCurrentDate(), games = [];
    gamesList.map(function(game){
      if (game.est.slice(0,8) < today && game.id.toString().slice(4,6)=='02') {
        games.push({
          id: misc.pad(game.id % 10000, 4), 
          date: game.est, 
          home: teams.shortFromScoreboardShort(game.h), 
          away: teams.shortFromScoreboardShort(game.a)
        });
      }
    });
    return games;
  })
}

var seasonDateArrayBuilder = function(season) {
  var scheduleURL = 'http://live.nhl.com/GameData/SeasonSchedule-'+season+'.json';

  return Request({
    method: 'GET',
    host: 'live.nhl.com',
    port: 80,
    path: scheduleURL
  }).then(function(data) {
    var gamesList = JSON.parse(data.body)
      , today = misc.getCurrentDate()
      , dates = []
      , gamesAtDate = []
      , date = 0
      , i = 0
      , len = gamesList.length
      , game
      , retObject = { dates: [] };

    for (; i < len; i++) {
      game = gamesList[i], date = game.est.slice(0,8);
      if (!retObject[date]){
        retObject[date] = [];
        dates.push(date);
      }
      if (game.id.toString().slice(4,6)=='03' || game.id.toString().slice(4,6)=='04' || today <= date) 
        continue;

      retObject[date].push({gameID: game.id, away: game.a, home: game.h});
    }

    dates.map(function(date){
      if(retObject[date].length < 1) {
        delete retObject[date];
      } else {
        retObject.dates.push(date);
      }
    })

    return retObject;
  })
}

exports.getTeamGameLists = function(season) {
  var teams = exports.getTeamsFor(season), 
      schedule = { teams: teams };

  teams.forEach(function(team){ schedule[team] = []; })
  
  return getDB()
  .then(function(db){
    return db.queryAsync(
      "SELECT * FROM AltGames "
      +"WHERE gameID > "+season.slice(0,4)+"020000 AND gameID < "+season.slice(0,4)+"030000 "
      +"ORDER BY date"
    )
  })
  .then(function(result){
    result.forEach(function(game) {
      schedule[game.team].push(game.gameID);
      if (typeof schedule[game.gameID] != 'object') { schedule[game.gameID] = {}; }
      schedule[game.gameID][game.team] = schedule[game.team].length-1;
    })

    return schedule;
  })
}

exports.getDates = function(season) {
  return seasonDateArrayBuilder(season);
}

exports.get = function(season) {
  return season < 20112012 ? Promise.resolve(oldSeasonScheduleArrayBuilder(season)) : seasonScheduleArrayBuilder(season);
}

if (require.main === module) {
  exports.getGamesFor('20152016').then(console.log);
}